/*
 * *********************************************************
 * Copyright (c) 2009 - 2014, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Nov 22, 2014
 * Author(s): FelixB <bayer.fel@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense;

import java.awt.Color;
import java.util.List;

import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.BaseAiFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.math.GeoMath;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.TacticalField;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.IDrawableShape;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.circle.Circle;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.circle.DrawableCircle;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.line.DrawableLine;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.line.ILine;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.line.Line;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.AIConfig;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.Geometry;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.ACalculator;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.algorithms.SimpleFoeBotCalc;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.algorithms.interfaces.IFoeBotCalc;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.data.FoeBotData;
import edu.dhbw.mannheim.tigers.sumatra.util.config.Configurable;
import edu.dhbw.mannheim.tigers.sumatra.view.visualizer.internals.field.EDrawableShapesLayer;


/**
 * This class provides a calculator to calculate important points to defend versus the foe bots.
 * 
 * @author FelixB <bayer.fel@gmail.com>
 */
public class DefenseCalc extends ACalculator
{
	
	private int						intersecPointDrawRadius		= 50;
	
	private final IFoeBotCalc	foeBotCalc;
	
	@Configurable(comment = "Distance to marker")
	private static float			distance2Marker				= 200;
	@Configurable
	private static float			distance2MarkerOffenseNear	= 500;
	
	@Configurable(comment = "Additional offset to penaltyArea margin")
	private static float			penaltyMarginOffset			= 70;
	
	@Configurable(comment = "Time [s] - the position of the future ball will be used to determine the danger from the ball position")
	private static float			ballLookaheadOfDefenders	= 1f;
	
	
	/**
	 * 
	 */
	public DefenseCalc()
	{
		foeBotCalc = new SimpleFoeBotCalc();
	}
	
	
	@Override
	public void doCalc(final TacticalField newTacticalField, final BaseAiFrame baseAiFrame)
	{
		List<FoeBotData> foeBotDataList = foeBotCalc.getFoeBotData(baseAiFrame);
		
		newTacticalField.setDangerousFoeBots(foeBotDataList);
		
		if (AIConfig.getGeometry().getOurHalf()
				.isPointInShape(baseAiFrame.getWorldFrame().getBall().getPosByTime(ballLookaheadOfDefenders)))
		{
			newTacticalField.setNeedTwoForBallBlock(true);
		} else
		{
			newTacticalField.setNeedTwoForBallBlock(false);
		}
		
		// drawings
		for (FoeBotData curFoeBotData : foeBotDataList)
		{
			ILine line = new Line(curFoeBotData.getBot().getPos(), curFoeBotData.getVectorGoalOurMidBot().multiplyNew(
					-1));
			IDrawableShape drawableLine = new DrawableLine(line, Color.BLACK);
			newTacticalField.getDrawableShapes().get(EDrawableShapesLayer.DEFENSE).add(drawableLine);
			
			for (IVector2 intersecPoint : curFoeBotData.getPassIntersections())
			{
				IDrawableShape drawableCircle = new DrawableCircle(new Circle(intersecPoint, intersecPointDrawRadius),
						Color.RED);
				newTacticalField.getDrawableShapes().get(EDrawableShapesLayer.DEFENSE).add(drawableCircle);
			}
			
			IDrawableShape drawableCircle = new DrawableCircle(new Circle(curFoeBotData.getNearestPointToOtherBot(),
					intersecPointDrawRadius),
					Color.BLUE);
			newTacticalField.getDrawableShapes().get(EDrawableShapesLayer.DEFENSE).add(drawableCircle);
			
			drawableCircle = new DrawableCircle(new Circle(curFoeBotData.getPointNearestToOurGoal(),
					intersecPointDrawRadius),
					Color.BLUE);
			newTacticalField.getDrawableShapes().get(EDrawableShapesLayer.DEFENSE).add(drawableCircle);
		}
		
		// draw a line from the ball to the center of the goal and to each goal post
		IVector2 ballPos = baseAiFrame.getWorldFrame().getBall().getPos();
		
		IVector2 leftPost = AIConfig.getGeometry().getGoalOur().getGoalPostLeft();
		IVector2 rightPost = AIConfig.getGeometry().getGoalOur().getGoalPostRight();
		IVector2 bisectorBall2Posts = GeoMath.calculateBisector(ballPos, leftPost, rightPost);
		
		DrawableLine ballToGoalCenter = new DrawableLine(Line.newLine(ballPos, bisectorBall2Posts));
		DrawableLine ballToLeftPost = new DrawableLine(Line.newLine(ballPos, AIConfig.getGeometry().getGoalOur()
				.getGoalPostLeft()));
		DrawableLine ballToRightPost = new DrawableLine(Line.newLine(ballPos, AIConfig.getGeometry().getGoalOur()
				.getGoalPostRight()));
		newTacticalField.getDrawableShapes().get(EDrawableShapesLayer.DEFENSE).add(ballToGoalCenter);
		newTacticalField.getDrawableShapes().get(EDrawableShapesLayer.DEFENSE).add(ballToLeftPost);
		newTacticalField.getDrawableShapes().get(EDrawableShapesLayer.DEFENSE).add(ballToRightPost);
	}
	
	
	/**
	 * @return
	 */
	public static float getPenaltyAreaMargin()
	{
		return Geometry.getPenaltyAreaMargin() + penaltyMarginOffset;
	}
	
	
	/**
	 * @return
	 */
	public static float getDistance2Marker()
	{
		return distance2Marker;
	}
	
	
	/**
	 * @return
	 */
	public static float getDistance2MarkerOffenseNear()
	{
		return distance2MarkerOffenseNear;
	}
}
