/*
 * *********************************************************
 * Copyright (c) 2009 - 2015, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Apr 1, 2015
 * Author(s): FelixB <bayer.fel@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.algorithms;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.BaseAiFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.math.AiMath;
import edu.dhbw.mannheim.tigers.sumatra.model.data.math.GeoMath;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.Vector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.TrackedTigerBot;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.ids.BotIDMap;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.ids.IBotIDMap;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.AIConfig;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.Geometry;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.DefenseCalc;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.algorithms.interfaces.IFoeBotCalc;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.data.DefensePoint;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.data.FoeBotData;


/**
 * Calculate an evaluation for each foe bot.
 * Returns a sorted list
 * 
 * @author FelixB <bayer.fel@gmail.com>
 */
public class SimpleFoeBotCalc implements IFoeBotCalc
{
	
	// private final Goal goalOur = AIConfig.getGeometry().getGoalOur();
	
	private float			intersecTolerance	= 2 * AIConfig.getGeometry().getBotRadius();
	
	// private float penaltyAreaTolerance = 1.2f * AIConfig.getGeometry().getBotRadius();
	
	private static float	penAreaMargin		= Geometry.getPenaltyAreaMargin();
	
	
	@Override
	public List<FoeBotData> getFoeBotData(final BaseAiFrame baseAiFrame)
	{
		ArrayList<FoeBotData> foeBotDataList = new ArrayList<FoeBotData>();
		Map<TrackedTigerBot, List<IVector2>> foeBotShootingMapping = new HashMap<TrackedTigerBot, List<IVector2>>();
		
		IBotIDMap<TrackedTigerBot> botsForNearestTest = new BotIDMap<TrackedTigerBot>();
		
		// create a FoeBotData object for every foe bot
		// initialize the pass mapping
		for (TrackedTigerBot curFoeBot : baseAiFrame.getWorldFrame().getFoeBots().values())
		{
			botsForNearestTest.put(curFoeBot.getId(), curFoeBot);
			if (curFoeBot.getId().equals(baseAiFrame.getKeeperFoeId()))
			{
				continue;
			}
			if (!AIConfig.getGeometry().getFieldWBorders().isPointInShape(curFoeBot.getPos()))
			{
				continue;
			}
			// if (curFoeBot.getPos().x() > (AIConfig.getGeometry().getFieldLength() / 4))
			// {
			// continue;
			// }
			IVector2 ballPos = baseAiFrame.getWorldFrame().getBall().getPos();
			IVector2 ball2Bot = curFoeBot.getPos().subtractNew(ballPos);
			
			IVector2 nearest2Ball = ballPos.addNew(ballPos.subtractNew(curFoeBot.getPos()).scaleToNew(
					-AIConfig.getGeometry().getBotRadius()));
			IVector2 nearest2marker = curFoeBot.getPos().addNew(ballPos.subtractNew(curFoeBot.getPos()).scaleToNew(
					AIConfig.getGeometry().getBotRadius() * 2));
			nearest2marker = AIConfig
					.getGeometry()
					.getPenaltyAreaOur()
					.nearestPointOutside(nearest2marker, baseAiFrame.getWorldFrame().getBall().getPos(),
							penAreaMargin);
			
			nearest2marker = AIConfig.getGeometry().getField().nearestPointInside(nearest2marker);
			// getNearest2Marker(baseAiFrame, ball2Bot, nearest2Ball);
			// MovementCon.assertDestinationValid(baseAiFrame.getWorldFrame(),
			// null, nearest2marker, false, false);
			// MovementCon.assertDestinationValid(baseAiFrame.getWorldFrame(),
			// null, nearest2Ball, false, false);
			
			foeBotDataList.add(new FoeBotData(curFoeBot, ball2Bot, nearest2marker,
					nearest2Ball));
			foeBotShootingMapping.put(curFoeBot, new ArrayList<IVector2>());
		}
		TrackedTigerBot ballPosessingBot = null;
		if (!botsForNearestTest.isEmpty())
		{
			IVector2 ballPos = baseAiFrame.getWorldFrame().getWorldFramePrediction().getBall().getPosAt(0.3f);
			ballPosessingBot = AiMath.getNearestBot(botsForNearestTest, ballPos);
			
			if (GeoMath.distancePP(ballPosessingBot.getPos(), ballPos) > 1000)
			{
				ballPosessingBot = null;
			}
		}
		
		
		// List<FoeBotData> botDataCopy = new ArrayList<>(foeBotDataList);
		// for (FoeBotData fbd : botDataCopy)
		// {
		// if (foeBotDataList.size() < 3)
		// {
		// break;
		// }
		// if (fbd.getBot().getPos().x() > (AIConfig.getGeometry().getFieldLength() / 4))
		// {
		// foeBotDataList.remove(fbd);
		// }
		// }
		
		// create a mapping for the shooting possibilities between foe bots
		for (int i = 0; i < (foeBotDataList.size() - 1); i++)
		{
			IVector2 shootingVector = AIConfig.getGeometry().getGoalOur().getGoalCenter()
					.subtractNew(foeBotDataList.get(i).getBot().getPos());
			foeBotShootingMapping.get(foeBotDataList.get(i).getBot()).add(shootingVector);
		}
		
		Vector2 lineIntersection = null;
		IVector2 ballPos = baseAiFrame.getWorldFrame().getBall().getPos();
		// calculate the intersection between pass paths and shoot path for every foe bot
		for (FoeBotData curFoeBotData : foeBotDataList)
		{
			for (FoeBotData shootingFoeBotData : foeBotDataList)
			{
				if (shootingFoeBotData.getBot().equals(curFoeBotData.getBot()))
				{
					continue;
				}
				for (IVector2 shootVector : foeBotShootingMapping.get(shootingFoeBotData.getBot()))
				{
					lineIntersection = GeoMath.intersectionPointPath(
							shootingFoeBotData.getBot().getPos(),
							shootVector,
							curFoeBotData.getBot().getPos(),
							ballPos.subtractNew(curFoeBotData.getBot().getPos())
							);
					if ((lineIntersection != null)
							&&
							!lineIntersection.equals(curFoeBotData.getBot().getPos(), intersecTolerance)
							&&
							!AIConfig.getGeometry().getPenaltyAreaOur()
									.isPointInShape(lineIntersection, DefenseCalc.getPenaltyAreaMargin()))
					{
						curFoeBotData.getPassIntersections().add(
								new DefensePoint(lineIntersection, new FoeBotData(curFoeBotData.getBot(), null, null, null)));
					}
				}
			}
			if ((ballPosessingBot != null) && (curFoeBotData.getBot().getId() == ballPosessingBot.getId()))
			// ||
			// (GeoMath.distancePL(curFoeBotData.getBot().getPos(), baseAiFrame.getWorldFrame().getBall().getPos(),
			// AIConfig.getGeometry().getGoalOur().getGoalCenter()) < (AIConfig.getGeometry().getBotRadius() * 2)))
			{
				curFoeBotData.setPosessesBall(true);
			}
		}
		
		foeBotDataList.sort(FoeBotData.DANGER_COMPARATOR);
		
		return foeBotDataList;
	}
	
	// private IVector2 getNearest2Marker(final BaseAiFrame frame, final IVector2 vectorGoalOurMidBot,
	// final IVector2 nearest2Ball)
	// {
	// IVector2 retVal = null;
	//
	// float distance = DefenseCalc.getDistance2Marker();
	// List<ARole> offRoles = frame.getPrevFrame().getPlayStrategy().getActiveRoles(ERole.OFFENSIVE);
	// for (ARole role : offRoles)
	// {
	// if (role.getBotID().isBot())
	// {
	// TrackedTigerBot bot = frame.getWorldFrame().getTiger(role.getBotID());
	// if (bot != null)
	// {
	// if (GeoMath.distancePP(bot, frame.getWorldFrame().getBall().getPos()) < 500)
	// {
	// distance = DefenseCalc.getDistance2MarkerOffenseNear();
	// }
	// }
	// }
	// }
	//
	//
	// IVector2 p = AIConfig.getGeometry().getGoalOur().getGoalCenter()
	// .addNew(vectorGoalOurMidBot.scaleToNew(vectorGoalOurMidBot.getLength2()
	// - ((2 * AIConfig.getGeometry().getBotRadius()) + distance)));
	//
	// float dist2Goal = GeoMath.distancePP(p, AIConfig.getGeometry().getGoalOur().getGoalCenter());
	// float dist2Nearest = GeoMath.distancePP(nearest2Ball, AIConfig.getGeometry().getGoalOur()
	// .getGoalCenter());
	// if (dist2Goal < dist2Nearest)
	// {
	// retVal = nearest2Goal;
	// }
	// retVal = p;
	//
	// retVal = AIConfig.getGeometry().getPenaltyAreaTheir()
	// .nearestPointOutside(retVal, goalOur.getGoalCenter(), penaltyAreaTolerance);
	// return AIConfig
	// .getGeometry()
	// .getPenaltyAreaOur()
	// .nearestPointOutside(retVal, goalOur.getGoalCenter().addNew(vectorGoalOurMidBot), penaltyAreaTolerance);
	// }
}
