/*
 * *********************************************************
 * Copyright (c) 2009 - 2014, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Nov 22, 2014
 * Author(s): FelixB <bayer.fel@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.data;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import com.sleepycat.persist.model.Persistent;

import edu.dhbw.mannheim.tigers.sumatra.model.data.math.GeoMath;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.ATrackedObject;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.AIConfig;


/**
 * FoeBotData
 * 
 * @author FelixB <bayer.fel@gmail.com>
 */
@Persistent(version = 1)
public class FoeBotData
{
	/**  */
	public static final Comparator<? super FoeBotData>	DANGER_COMPARATOR	= new DangerComparator();
	
	private final ATrackedObject								foeBot;
	
	/** Vector from the center of our goal to the foe bot */
	private final IVector2										vectorGoalOurMidBot;
	/** Intersections of vectorGoalOurMidBot and a vector between foe bots */
	private final List<IVector2>								passIntersections	= new ArrayList<IVector2>();
	private final IVector2										nearest2Marker;
	private final IVector2										nearest2Goal;
	private boolean												posessesBall		= false;
	
	
	@SuppressWarnings("unused")
	private FoeBotData()
	{
		// berkeley support
		foeBot = null;
		vectorGoalOurMidBot = null;
		nearest2Marker = null;
		nearest2Goal = null;
	}
	
	
	/**
	 * @param foeBot
	 * @param vectorGoalOurMidBot
	 * @param nearest2Marker
	 * @param nearest2Goal
	 */
	public FoeBotData(final ATrackedObject foeBot, final IVector2 vectorGoalOurMidBot, final IVector2 nearest2Marker,
			final IVector2 nearest2Goal)
	{
		this.foeBot = foeBot;
		this.vectorGoalOurMidBot = vectorGoalOurMidBot;
		this.nearest2Marker = nearest2Marker;
		this.nearest2Goal = nearest2Goal;
	}
	
	
	/**
	 * @return the foeBot
	 */
	public ATrackedObject getBot()
	{
		return foeBot;
	}
	
	
	/**
	 * @return the vectorGoalOurMidBot
	 */
	public IVector2 getVectorGoalOurMidBot()
	{
		return vectorGoalOurMidBot;
	}
	
	
	/**
	 * @return
	 */
	public IVector2 getPointNearestToOurGoal()
	{
		return nearest2Goal;
	}
	
	
	/**
	 * @return
	 */
	public IVector2 getNearestPointToOtherBot()
	{
		return nearest2Marker;
	}
	
	
	/**
	 * @return the passIntersections
	 */
	public List<IVector2> getPassIntersections()
	{
		return passIntersections;
	}
	
	
	/**
	 * @return the posessesBall
	 */
	public boolean posessesBall()
	{
		return posessesBall;
	}
	
	
	/**
	 * @param posessesBall the posessesBall to set
	 */
	public void setPosessesBall(final boolean posessesBall)
	{
		this.posessesBall = posessesBall;
	}
	
	@SuppressWarnings("unused")
	private static class DistanceComparator implements Comparator<FoeBotData>
	{
		@Override
		public int compare(final FoeBotData bot1, final FoeBotData bot2)
		{
			if (bot1.posessesBall())
			{
				return 1;
			} else if (bot2.posessesBall())
			{
				return -1;
			} else
			{
				float distBot1 = GeoMath.distancePP(bot1.getBot(), AIConfig.getGeometry().getGoalOur().getGoalCenter());
				float distBot2 = GeoMath.distancePP(bot2.getBot(), AIConfig.getGeometry().getGoalOur().getGoalCenter());
				return (int) Math.signum(distBot1 - distBot2);
			}
		}
	}
	
	@SuppressWarnings("unused")
	private static class DangerComparator implements Comparator<FoeBotData>
	{
		private static float	halfFieldWidth		= AIConfig.getGeometry().getFieldWidth() / 2f;
		
		/**
		 * The factor a bot becomes more dangerous if he is in the center of the field y wise.
		 */
		private static float	fieldWidthFactor	= 1.2f;
		
		
		@Override
		public int compare(final FoeBotData bot1, final FoeBotData bot2)
		{
			if (bot1.posessesBall())
			{
				return 1;
			} else if (bot2.posessesBall())
			{
				return -1;
			} else
			{
				float distBot1 = GeoMath.distancePP(bot1.getBot(), AIConfig.getGeometry().getGoalOur().getGoalCenter());
				float distBot2 = GeoMath.distancePP(bot2.getBot(), AIConfig.getGeometry().getGoalOur().getGoalCenter());
				
				distBot1 *= (((1f - fieldWidthFactor) / Math.abs(halfFieldWidth)) * bot1.getBot().getPos().y())
						+ fieldWidthFactor;
				distBot2 *= (((1f - fieldWidthFactor) / Math.abs(halfFieldWidth)) * bot2.getBot().getPos().y())
						+ fieldWidthFactor;
				
				return (int) Math.signum(distBot1 - distBot2);
			}
		}
	}
}
