/*
 * *********************************************************
 * Copyright (c) 2009 - 2014, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Oct 7, 2014
 * Author(s): Mark Geiger <Mark.Geiger@dlr.de>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.general;

import java.util.List;

import matlabcontrol.MatlabConnectionException;
import matlabcontrol.MatlabInvocationException;
import matlabcontrol.MatlabProxy;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.BaseAiFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.math.GeoMath;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.ETeamColor;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.TacticalField;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.TrackedBall;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.ACalculator;
import edu.dhbw.mannheim.tigers.sumatra.util.MatlabConnection;
import edu.dhbw.mannheim.tigers.sumatra.util.clock.SumatraClock;
import edu.dhbw.mannheim.tigers.sumatra.util.config.Configurable;
import edu.dhbw.mannheim.tigers.sumatra.util.csvexporter.BallWatcher;
import edu.dhbw.mannheim.tigers.sumatra.util.csvexporter.CSVExporter;
import edu.dhbw.mannheim.tigers.sumatra.util.csvexporter.IBallWatcherObserver;


/**
 * log ball data to help future ballMovementPredictions
 * 
 * @author: Mark Geiger <Mark.Geiger@dlr.de>
 */
public class BallMovementAnalyserCalc extends ACalculator implements IBallWatcherObserver
{
	/*
	 * How To use this Calculator:
	 * 1) Enable learnBallMovement via ConfigEditor.
	 * 2) Use a Bot to Kick ball from one corner to another. Ball should stand still when he reaches the "other" corner.
	 * 3) Have Matlab installed!
	 * 4) BallMovementAnalyser function will then automatically be updated in "TrackedBall"
	 * 5) Check if new generated equations are somehow broken :P
	 * 6) Generated Values are used for methods like "getPosAt(float s)" in TrackedBall.
	 */
	
	
	@Configurable(comment = "Bool - Export move data to a csv file")
	private static boolean			learnBallMovement		= false;
	
	private static final Logger	log						= Logger.getLogger(BallMovementAnalyserCalc.class
																				.getName());
	private boolean					measurementRunning	= false;
	
	private BallWatcher				ballWatcher				= null;
	
	
	/**
	 * 
	 */
	public BallMovementAnalyserCalc()
	{
	}
	
	
	@Override
	public void doCalc(final TacticalField newTacticalField, final BaseAiFrame baseAiFrame)
	{
		if (learnBallMovement && !measurementRunning && (baseAiFrame.getTeamColor() == ETeamColor.YELLOW))
		{
			log.info("started recording BallMovementData");
			measurementRunning = true;
			String name = "ballMovement" + SumatraClock.currentTimeMillis();
			ballWatcher = new BallWatcher(name);
			ballWatcher.addObserver(this);
			ballWatcher.start();
		}
		if (!learnBallMovement)
		{
			measurementRunning = false;
		}
	}
	
	
	@Override
	public void beforeExport(final CSVExporter exporter)
	{
	}
	
	
	@Override
	public void onAddCustomData(final List<Object> data)
	{
		data.add(GeoMath.distancePP(ballWatcher.getLastBall().getPos().getXYVector(), ballWatcher.getInitBallPos()));
	}
	
	
	@Override
	public void postProcessing(final String fileName)
	{
		MatlabProxy mp;
		try
		{
			mp = MatlabConnection.getMatlabProxy();
			Object[] values = mp.returningFeval("ballMovementAnalyser", 1, fileName);
			double[] coeffs = (double[]) values[0];
			TrackedBall.setBallMovementParameters((float) coeffs[0], (float) coeffs[1], (float) coeffs[2],
					(float) coeffs[3], (float) coeffs[4], (float) coeffs[5]);
		} catch (MatlabConnectionException err)
		{
			log.error(err.getMessage(), err);
		} catch (MatlabInvocationException err)
		{
			log.error("Error evaluating matlab function: " + err.getMessage(), err);
		}
	}
}
