/*
 * *********************************************************
 * Copyright (c) 2009 - 2013, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: 12.12.2014
 * Author(s): MarkG
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.offense;

import java.awt.Color;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sumatra.model.data.Referee.SSL_Referee.Command;
import edu.dhbw.mannheim.tigers.sumatra.model.data.airecord.IRecordFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.BaseAiFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.WorldFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.math.GeoMath;
import edu.dhbw.mannheim.tigers.sumatra.model.data.math.OffensiveMath;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.EGameState;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.ETeamColor;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.OffensiveStrategy;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.OffensiveStrategy.EOffensiveStrategy;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.SpecialMoveCommand;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.TacticalField;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.triangle.DrawableTriangle;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.Vector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.ids.BotID;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.AIConfig;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.ACalculator;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.offense.data.OffensiveMovePosition.EOffensiveMoveType;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.ARole;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.ERole;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.bots.ABot;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.bots.TigerBotV3;
import edu.dhbw.mannheim.tigers.sumatra.util.clock.SumatraClock;
import edu.dhbw.mannheim.tigers.sumatra.view.visualizer.internals.field.EDrawableShapesLayer;


/**
 * Calculates OffenseStrategy for the OffenseRole.
 * 
 * @author Mark Geiger <Mark.Geiger@dlr.de>
 */
public class OffensiveStrategyCalc extends ACalculator
{
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	
	protected static final Logger	log							= Logger
																					.getLogger(OffensiveStrategyCalc.class.getName());
	private long						cheeringTimer				= 0;
	
	private OffensiveStrategy		offensiveStrategy			= new OffensiveStrategy();
	
	private boolean					waitForNormalStart		= false;
	
	private boolean					kickoff						= false;
	
	private long						delayTimer					= 0;
	
	private float						waitTime						= 0;
	
	private boolean					waitForHelperResponse	= false;
	
	private SpecialMoveCommand		prevCommand					= null;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// -------------------------------------- ------------------------------------
	
	
	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	
	@Override
	public void doCalc(final TacticalField newTacticalField, final BaseAiFrame baseAiFrame)
	{
		offensiveStrategy = new OffensiveStrategy();
		calculateOffensiveStrategy(newTacticalField, baseAiFrame);
	}
	
	
	private void calculateOffensiveStrategy(final TacticalField newTacticalField, final BaseAiFrame baseAiFrame)
	{
		for (BotID key : baseAiFrame.getWorldFrame().tigerBotsAvailable.keySet())
		{
			offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.GET);
		}
		
		BotID bestGetter = OffensiveMath.getBestGetter(baseAiFrame, newTacticalField);
		BotID bestRedirector = OffensiveMath.getBestRedirector(baseAiFrame.getWorldFrame(), newTacticalField);
		
		if (baseAiFrame.getPrevFrame() != null)
		{
			if (baseAiFrame.getPrevFrame().getPlayStrategy() != null)
			{
				if (baseAiFrame.getPrevFrame().getPlayStrategy().getActiveRoles(ERole.OFFENSIVE) != null)
				{
					for (ARole role : baseAiFrame.getPrevFrame().getPlayStrategy().getActiveRoles(ERole.OFFENSIVE))
					{
						if (role.getBotID() == bestRedirector)
						{
							if ((role.getCurrentState() == EOffensiveStrategy.KICK) &&
									(GeoMath.distancePP(role.getPos(), baseAiFrame.getWorldFrame().getBall().getPos()) < 500f))
							{
								bestRedirector = null;
							}
						}
					}
				}
			}
		}
		
		if (bestRedirector != null)
		{
			// alle potential redirector zu dem hier setzten.
			offensiveStrategy.getDesiredBots().clear();
			offensiveStrategy.getDesiredBots().add(bestRedirector);
			offensiveStrategy.getCurrentOffensivePlayConfiguration().put(bestRedirector,
					EOffensiveStrategy.REDIRECT_CATCH_SPECIAL_MOVE);
			offensiveStrategy.setMinNumberOfBots(1);
			offensiveStrategy.setMaxNumberOfBots(1);
		} else if (bestGetter != null)
		{
			offensiveStrategy.getDesiredBots().clear();
			offensiveStrategy.getDesiredBots().add(bestGetter);
			offensiveStrategy.setMinNumberOfBots(0);
			offensiveStrategy.setMaxNumberOfBots(1);
		}
		
		if (baseAiFrame.getPrevFrame().getPlayStrategy().getActiveRoles() != null)
		{
			// check if more than 1 offensive bot is needed ( passing, kickoff, ... )
			addSecondaryOffensiveBots(newTacticalField, baseAiFrame);
			
			// change States of current offensiveBots if a change is needed
			calculateActiveStateChanges(newTacticalField, baseAiFrame);
		}
		
		// read currentGameState and adjust/change current States of all offensiveRoles
		reactOnRefSignals(newTacticalField, baseAiFrame);
		keepActiveStatus(baseAiFrame.getWorldFrame(), baseAiFrame, newTacticalField, newTacticalField.getGameState());
		
		if (AIConfig.getGeometry().getPenaltyAreaOur()
				.isPointInShape(baseAiFrame.getWorldFrame().getBall().getPos(), 300) ||
				!AIConfig.getGeometry().getField().isPointInShape(baseAiFrame.getWorldFrame().getBall().getPos()))
		{
			offensiveStrategy.setMaxNumberOfBots(0);
			offensiveStrategy.setMinNumberOfBots(0);
			offensiveStrategy.getDesiredBots().clear();
		}
		
		newTacticalField.setOffensiveStrategy(offensiveStrategy);
		
		// This is really important!
		cheerWhenWeShootAGoal(newTacticalField, baseAiFrame);
	}
	
	
	private void gameStateChange(final WorldFrame wFrame, final EGameState from, final EGameState to)
	{
		switch (to)
		{
			case DIRECT_KICK_THEY:
			case THROW_IN_THEY:
				for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
				{
					offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.INTERCEPT);
				}
				break;
			case CORNER_KICK_THEY:
			case TIMEOUT_THEY:
			case TIMEOUT_WE:
			case GOAL_KICK_THEY:
			case HALTED:
			case PREPARE_PENALTY_THEY:
			case PREPARE_KICKOFF_THEY:
			case PREPARE_PENALTY_WE:
			case STOPPED:
				kickoff = false;
				// planedStrategy = EOffensiveStrategy.STOP;
				// waitingForNormalStart = false;
				for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
				{
					offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.STOP);
				}
				break;
			case PREPARE_KICKOFF_WE:
				for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
				{
					offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.GET);
				}
				// inKickoff = true;
				kickoff = true;
				// delayTimer = SumatraClock.currentTimeMillis();
				// planedStrategy = EOffensiveStrategy.DELAY;
				waitForNormalStart = true;
				break;
			case CORNER_KICK_WE:
			case GOAL_KICK_WE:
			case THROW_IN_WE:
				// forcePass = true;
			case DIRECT_KICK_WE:
				
				delayTimer = SumatraClock.nanoTime();
				waitForNormalStart = false;
				for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
				{
					offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.DELAY);
				}
				waitTime = OffensiveConstants.getDelayWaitTime();
				
				break;
			case RUNNING:
				delayTimer = 0;
				// kickOffHelperBot = null;
				kickoff = false;
				waitForNormalStart = false;
				for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
				{
					offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.GET);
				}
				break;
			default:
				break;
		}
	}
	
	
	private void reactOnRefSignals(final TacticalField newTacticalField, final BaseAiFrame baseAiFrame)
	{
		WorldFrame wFrame = baseAiFrame.getWorldFrame();
		EGameState gameState = newTacticalField.getGameState();
		EGameState preGameState = baseAiFrame.getPrevFrame().getTacticalField().getGameState();
		
		if (!preGameState.equals(gameState))
		{
			gameStateChange(wFrame, preGameState, gameState);
		}
		
		if (baseAiFrame.getNewRefereeMsg() != null)
		{
			if (baseAiFrame.getNewRefereeMsg().getCommand().equals(Command.NORMAL_START))
			{
				for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
				{
					offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.GET);
				}
			}
		}
	}
	
	
	private void keepActiveStatus(final WorldFrame wFrame, final BaseAiFrame baseAiFrame,
			final TacticalField newTacticalField, final EGameState gameState)
	{
		switch (gameState)
		{
		
			case THROW_IN_THEY:
			case DIRECT_KICK_THEY:
				for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
				{
					offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.INTERCEPT);
				}
				offensiveStrategy.getDesiredBots().clear();
				offensiveStrategy.getDesiredBots().add(null);
				offensiveStrategy.setMaxNumberOfBots(1);
				break;
			case CORNER_KICK_THEY:
			case TIMEOUT_THEY:
			case TIMEOUT_WE:
			case GOAL_KICK_THEY:
			case HALTED:
			case PREPARE_PENALTY_THEY:
			case PREPARE_KICKOFF_THEY:
			case PREPARE_PENALTY_WE:
				offensiveStrategy.getDesiredBots().clear();
				offensiveStrategy.setMaxNumberOfBots(0);
				offensiveStrategy.setMinNumberOfBots(0);
				break;
			case STOPPED:
				for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
				{
					offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.STOP);
				}
				// no offensibe in stop.
				// offensiveStrategy.getDesiredBots().clear();
				offensiveStrategy.setMaxNumberOfBots(1);
				offensiveStrategy.setMinNumberOfBots(0);
				break;
			case PREPARE_KICKOFF_WE:
				if (waitForNormalStart)
				{
					offensiveStrategy.getDesiredBots().clear();
					offensiveStrategy.getDesiredBots().add(null);
					offensiveStrategy.setMaxNumberOfBots(1);
					offensiveStrategy.getCurrentOffensivePlayConfiguration().clear();
					offensiveStrategy.getUnassignedStrategies().add(EOffensiveStrategy.STOP);
				}
				break;
			case CORNER_KICK_WE:
			case GOAL_KICK_WE:
			case THROW_IN_WE:
			case DIRECT_KICK_WE:
				// if (delayTimer)
				if ((waitTime != 0) && (((SumatraClock.nanoTime() - delayTimer) * 1e-9) < waitTime))
				{
					for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
					{
						offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.DELAY);
					}
					offensiveStrategy.getDesiredBots().clear();
					offensiveStrategy.getDesiredBots().add(null);
					offensiveStrategy.setMaxNumberOfBots(1);
				}
				break;
			case RUNNING:
				// nothing todo here
				break;
			default:
				break;
		}
	}
	
	
	private void addSecondaryOffensiveBots(final TacticalField newTacticalField,
			final BaseAiFrame baseAiFrame)
	{
		IRecordFrame prevFrame = baseAiFrame.getPrevFrame();
		for (IVector2 pos : prevFrame.getAICom().getDelayMoves())
		{
			SpecialMoveCommand command = new SpecialMoveCommand();
			command.getMovePosition().add(pos);
			command.getMoveTimes().add(0f);
			command.setResponseStep(0);
			offensiveStrategy.getSpecialMoveCommands().add(command);
			offensiveStrategy.getUnassignedStrategies().add(EOffensiveStrategy.REDIRECT_CATCH_SPECIAL_MOVE);
			offensiveStrategy.getDesiredBots().add(null);
			offensiveStrategy.setMaxNumberOfBots(offensiveStrategy.getMaxNumberOfBots() + 1);
		}
		
		if ((prevFrame.getAICom().getOffensiveRolePassTargetID() != null) &&
				(newTacticalField.getGameState() != EGameState.PREPARE_KICKOFF_WE))
		{
			BotID passReciever = prevFrame.getAICom().getOffensiveRolePassTargetID();
			SpecialMoveCommand command = new SpecialMoveCommand();
			
			command.getMovePosition().add(prevFrame.getAICom().getOffensiveRolePassTarget());
			command.getMoveTimes().add(0f);
			command.setResponseStep(0);
			
			if (offensiveStrategy.getCurrentOffensivePlayConfiguration().containsKey(passReciever))
			{
				if (offensiveStrategy.getCurrentOffensivePlayConfiguration().get(passReciever) == EOffensiveStrategy.REDIRECT_CATCH_SPECIAL_MOVE)
				{
					if (prevCommand != null)
					{
						command = prevCommand;
					}
				}
			}
			
			offensiveStrategy.getDesiredBots().add(passReciever);
			offensiveStrategy.setMaxNumberOfBots(offensiveStrategy.getMaxNumberOfBots() + 1);
			offensiveStrategy.getSpecialMoveCommands().add(command);
			offensiveStrategy.getCurrentOffensivePlayConfiguration().put(passReciever,
					EOffensiveStrategy.REDIRECT_CATCH_SPECIAL_MOVE);
			prevCommand = command;
		}
		
		if (((kickoff == true) && (waitForNormalStart == false)))
		{
			for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
			{
				offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.GET);
			}
		}
	}
	
	
	private void calculateActiveStateChanges(final TacticalField newTacticalField,
			final BaseAiFrame baseAiFrame)
	{
		// change state of GET to KICK and KICK to GET
		checkBallObtained(newTacticalField, baseAiFrame);
		checkNormalStartCalled(newTacticalField, baseAiFrame);
		checkDelayCounter(newTacticalField, baseAiFrame);
	}
	
	
	private void checkNormalStartCalled(
			final TacticalField newTacticalField,
			final BaseAiFrame baseAiFrame)
	{
		if (baseAiFrame.getNewRefereeMsg() != null)
		{
			if ((baseAiFrame.getNewRefereeMsg().getCommand() == Command.NORMAL_START)
					&& (newTacticalField.getGameState() == EGameState.PREPARE_KICKOFF_WE))
			{
				waitForNormalStart = false;
				delayTimer = SumatraClock.nanoTime();
				for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
				{
					offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.GET);
				}
				// its the minimum waitTime.
				waitTime = OffensiveConstants.getDelayWaitTime(); // this time doesnt matter, because it gets triggered by
																					// the
			}
		}
	}
	
	
	private void checkDelayCounter(
			final TacticalField newTacticalField,
			final BaseAiFrame baseAiFrame)
	{
		if (baseAiFrame.getPrevFrame().getAICom().hasResponded()
		/* || (((SumatraClock.nanoTime() - kickOffTimer) * 1e-9) > kickOffTimeout) */)
		{
			waitForHelperResponse = false;
			if (newTacticalField.getGameState() == EGameState.PREPARE_KICKOFF_WE)
			{
				// specialKickoffTimer = SumatraClock.nanoTime();
			}
		}
		if ((waitTime != 0) && (waitForHelperResponse == false))
		{
			if (((SumatraClock.nanoTime() - delayTimer) * 1e-9) > waitTime)
			{
				waitTime = 0;
				for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
				{
					EOffensiveStrategy state = offensiveStrategy.getCurrentOffensivePlayConfiguration().get(key);
					if ((state == EOffensiveStrategy.DELAY))
					{
						offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.GET);
					}
				}
			}
		}
	}
	
	
	private void checkBallObtained(
			final TacticalField newTacticalField,
			final BaseAiFrame baseAiFrame)
	{
		IVector2 ballPos = baseAiFrame.getWorldFrame().getBall().getPos();
		IVector2 botPos = null;
		IVector2 destination = null;
		float distanceToBall = 655;
		float distanceToDestination = 740;
		float checkDistance = OffensiveConstants.getFinalKickStateDistance();
		float normalDist = 200f;
		
		// für alle current bots die in GET oder KICK sind
		for (BotID key : offensiveStrategy.getCurrentOffensivePlayConfiguration().keySet())
		{
			EOffensiveStrategy state = offensiveStrategy.getCurrentOffensivePlayConfiguration().get(key);
			if ((state != EOffensiveStrategy.GET) && (state != EOffensiveStrategy.KICK))
			{
				continue;
			}
			if (!baseAiFrame.getWorldFrame().getTigerBotsAvailable().containsKey(key))
			{
				continue;
			}
			if (AIConfig.getGeometry().getPenaltyAreaTheir().isPointInShape(ballPos))
			{
				continue;
			}
			
			botPos = baseAiFrame.getWorldFrame().getTigerBotsAvailable().get(key).getPos();
			destination = newTacticalField.getOffenseMovePositions().get(key);
			
			IVector2 botToBallNormal = ballPos.subtractNew(botPos).getNormalVector();
			IVector2 kt1 = botPos.addNew(botToBallNormal.multiplyNew(normalDist));
			IVector2 kt2 = botPos.addNew(botToBallNormal.multiplyNew(-normalDist));
			
			IVector2 bt1 = ballPos.addNew(botToBallNormal.multiplyNew(normalDist / 4f));
			IVector2 bt2 = ballPos.addNew(botToBallNormal.multiplyNew(-normalDist / 4f));
			
			bt1 = bt1.addNew(botPos.subtractNew(bt1).normalizeNew().multiplyNew(checkDistance));
			bt2 = bt2.addNew(botPos.subtractNew(bt2).normalizeNew().multiplyNew(checkDistance));
			
			DrawableTriangle tri = new DrawableTriangle(bt1, kt1, botPos, new Color(0, 50, 200, 20));
			DrawableTriangle tri2 = new DrawableTriangle(bt2, botPos, kt2, new Color(0, 50, 200, 20));
			DrawableTriangle tri3 = new DrawableTriangle(bt1, bt2, botPos, new Color(0, 50, 200, 20));
			tri.setFill(true);
			tri2.setFill(true);
			tri3.setFill(true);
			
			if ((state == EOffensiveStrategy.GET)
					&& (newTacticalField.getOffenseMovePositions() != null)
					&& (newTacticalField.getOffenseMovePositions().get(key) != null)
					&& (newTacticalField.getOffenseMovePositions().get(key).getType() != null))
			{
				EOffensiveMoveType type = newTacticalField.getOffenseMovePositions().get(key).getType();
				if (((GeoMath.distancePP(botPos, destination) < (distanceToDestination)) && (GeoMath.distancePP(botPos,
						ballPos) < (distanceToBall))) && (type != EOffensiveMoveType.UNREACHABLE))
				{
					offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.KICK);
				}
				
				boolean freeOfBots = true;
				
				for (BotID obstacle : baseAiFrame.getWorldFrame().getTigerBotsVisible().keySet())
				{
					if (tri.isPointInShape(baseAiFrame.getWorldFrame().getTigerBotsVisible().get(obstacle).getPos()) ||
							tri2.isPointInShape(baseAiFrame.getWorldFrame().getTigerBotsVisible().get(obstacle).getPos()) ||
							tri3.isPointInShape(baseAiFrame.getWorldFrame().getTigerBotsVisible().get(obstacle).getPos()))
					{
						if (obstacle != key)
						{
							freeOfBots = false;
							break;
						}
					}
				}
				for (BotID obstacle : baseAiFrame.getWorldFrame().getFoeBots().keySet())
				{
					if (tri.isPointInShape(baseAiFrame.getWorldFrame().getFoeBots().get(obstacle).getPos()) ||
							tri2.isPointInShape(baseAiFrame.getWorldFrame().getFoeBots().get(obstacle).getPos()) ||
							tri3.isPointInShape(baseAiFrame.getWorldFrame().getFoeBots().get(obstacle).getPos()))
					{
						if (obstacle != key)
						{
							freeOfBots = false;
							break;
						}
					}
				}
				if (freeOfBots)
				{
					offensiveStrategy.getCurrentOffensivePlayConfiguration().put(key, EOffensiveStrategy.KICK);
				}
				
				if (offensiveStrategy.getCurrentOffensivePlayConfiguration().containsKey(key))
				{
					if ((offensiveStrategy.getCurrentOffensivePlayConfiguration().get(key) == EOffensiveStrategy.KICK)
							&& (GeoMath.distancePP(botPos, ballPos) > (checkDistance + 50f)))
					{
						newTacticalField.getDrawableShapes().get(EDrawableShapesLayer.OFFENSIVE).add(tri);
						newTacticalField.getDrawableShapes().get(EDrawableShapesLayer.OFFENSIVE).add(tri2);
						newTacticalField.getDrawableShapes().get(EDrawableShapesLayer.OFFENSIVE).add(tri3);
					}
				}
			}
		}
		return;
	}
	
	
	private void cheerWhenWeShootAGoal(
			final TacticalField newTacticalField,
			final BaseAiFrame baseAiFrame)
	{
		if (baseAiFrame.getNewRefereeMsg() != null)
		{
			ETeamColor color = baseAiFrame.getWorldFrame().getTeamColor();
			if (color == ETeamColor.YELLOW)
			{
				if (baseAiFrame.getNewRefereeMsg().getCommand() == Command.GOAL_YELLOW)
				{
					cheeringTimer = SumatraClock.nanoTime();
					for (BotID key : baseAiFrame.getWorldFrame().tigerBotsAvailable.keySet())
					{
						ABot bot = baseAiFrame.getWorldFrame().tigerBotsAvailable.get(key).getBot();
						
						if (bot instanceof TigerBotV3)
						{
							((TigerBotV3) bot).setCheering(true);
						}
					}
				}
			}
			else
			{
				if (baseAiFrame.getNewRefereeMsg().getCommand() == Command.GOAL_BLUE)
				{
					cheeringTimer = SumatraClock.nanoTime();
					for (BotID key : baseAiFrame.getWorldFrame().tigerBotsAvailable.keySet())
					{
						ABot bot = baseAiFrame.getWorldFrame().tigerBotsAvailable.get(key).getBot();
						
						if (bot instanceof TigerBotV3)
						{
							((TigerBotV3) bot).setCheering(true);
						}
					}
				}
			}
		}
		if ((((SumatraClock.nanoTime() - cheeringTimer) * 1e-9) > OffensiveConstants.getCheeringStopTimer())
				&& (cheeringTimer != 0))
		{
			for (BotID key : baseAiFrame.getWorldFrame().tigerBotsAvailable.keySet())
			{
				ABot bot = baseAiFrame.getWorldFrame().tigerBotsAvailable.get(key).getBot();
				
				if (bot instanceof TigerBotV3)
				{
					((TigerBotV3) bot).setCheering(false);
				}
			}
			cheeringTimer = 0;
		}
	}
	
	
	// --------------------------------------------------------------------------
	// --- getter/setter --------------------------------------------------------
	// --------------------------------------------------------------------------
	
	
	/**
	 * @return the kickOffTarget
	 */
	public static IVector2 getKickOffTarget()
	{
		return new Vector2(
				(AIConfig.getGeometry().getFieldLength() / 4.5f),
				AIConfig.getGeometry().getFieldWidth() / 2.5f);
	}
	
	
	@Override
	public void fallbackCalc(final TacticalField newTacticalField, final BaseAiFrame baseAiFrame)
	{
		offensiveStrategy = new OffensiveStrategy();
		offensiveStrategy.setMinNumberOfBots(0);
		offensiveStrategy.setMaxNumberOfBots(1);
		offensiveStrategy.getUnassignedStrategies().add(EOffensiveStrategy.GET);
		offensiveStrategy.getDesiredBots().add(null);
		newTacticalField.setOffensiveStrategy(offensiveStrategy);
	}
}
