/*
 * *********************************************************
 * Copyright (c) 2009 - 2013, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: 12.12.2014
 * Author(s): MarkG
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.offense.data;

import com.sleepycat.persist.model.Persistent;

import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.WorldFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.math.GeoMath;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.TacticalField;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.Vector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.TrackedTigerBot;


/**
 * 
 */
@Persistent
public class OffensiveMovePosition extends Vector2
{
	
	
	/**
	 * @author MarkG
	 */
	public enum EOffensiveMoveType
	{
		/*** */
		NORMAL,
		/*** */
		IGNORE_BALL,
		/*** */
		UNREACHABLE;
	}
	
	private TrackedTigerBot		bot		= null;
	private float					scoring	= 0;
	private EOffensiveMoveType	type		= EOffensiveMoveType.NORMAL;
	
	
	/**
	  * 
	  */
	public OffensiveMovePosition()
	{
		
	}
	
	
	/**
	 * @param position
	 * @param bot
	 * @param type
	 */
	public OffensiveMovePosition(final IVector2 position, final TrackedTigerBot bot, final EOffensiveMoveType type)
	{
		super(position);
		this.bot = bot;
		this.type = type;
	}
	
	
	/**
	 * @return the scoring
	 */
	public float getScoring()
	{
		return scoring;
	}
	
	
	/**
	 * @param scoring
	 */
	public void setScoring(final float scoring)
	{
		this.scoring = scoring;
	}
	
	
	/**
	 * @return the bot
	 */
	public TrackedTigerBot getBot()
	{
		return bot;
	}
	
	
	/**
	 * @return the type
	 */
	public EOffensiveMoveType getType()
	{
		return type;
	}
	
	
	/**
	 * small scorings are good
	 * 
	 * @param wFrame
	 * @param tacticalField
	 */
	public void generateScoring(final WorldFrame wFrame, final TacticalField tacticalField)
	{
		float distanceToDestination = GeoMath.distancePP(this, bot.getPos());
		float distanceToBall = GeoMath.distancePP(wFrame.getBall().getPos(), this);
		scoring = (distanceToDestination / 2.0f) + (distanceToBall / 2.0f);
	}
}
