/*
 * *********************************************************
 * Copyright (c) 2009 - 2015, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Apr 5, 2015
 * Author(s): Nicolai Ommer <nicolai.ommer@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.plays.defense.algorithms;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.MetisAiFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.math.GeoMath;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.AIConfig;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.data.DefensePoint;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.data.FoeBotData;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.conditions.move.MovementCon;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.plays.defense.algorithms.helpers.IPointOnLine;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.plays.defense.algorithms.helpers.PassiveAgressivePointCalc;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.defense.DefenderRole;


/**
 * Make sure we drive towards a good point on the line as soon as we reached the line
 * 
 * @author Nicolai Ommer <nicolai.ommer@gmail.com>
 */
public class DriveOnLinePointCalc extends ShortestPathDefensePointCalc
{
	
	// private final IPointOnLine pointOnLineGetter = new PassIntersectionPointCalc();
	private final IPointOnLine	pointOnLineGetter		= new PassiveAgressivePointCalc();
	
	private final float			minDistShootVector	= AIConfig.getGeometry().getBotRadius() / 2f;
	
	
	@Override
	public Map<DefenderRole, DefensePoint> getDefenderDistribution(final MetisAiFrame frame,
			final List<DefenderRole> defenders,
			final List<FoeBotData> foeBotDataList)
	{
		
		Map<DefenderRole, DefensePoint> distribution = super.getDefenderDistribution(frame, defenders, foeBotDataList);
		DefenderRole curDefender = null;
		DefensePoint curDefPoint = null;
		
		for (Entry<DefenderRole, DefensePoint> entry : distribution.entrySet())
		{
			curDefender = entry.getKey();
			curDefPoint = entry.getValue();
			
			if (GeoMath.distancePP(curDefPoint, curDefender.getPos()) < minDistShootVector)
			{
				distribution.put(entry.getKey(), pointOnLineGetter.getPointOnLine(entry.getValue(), frame));
			}
		}
		
		distribution.forEach((defender, defPoint) -> MovementCon.assertDestinationValid(frame.getWorldFrame(),
				defender.getBot().getId(), defPoint, false, false));
		
		return distribution;
	}
}
