/*
 * *********************************************************
 * Copyright (c) 2009 - 2015, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Apr 5, 2015
 * Author(s): Nicolai Ommer <nicolai.ommer@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.plays.defense.algorithms;

import java.util.List;
import java.util.Map;

import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.MetisAiFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.math.GeoMath;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.circle.Circle;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.TrackedTigerBot;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.ids.BotID;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.AIConfig;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.data.DefensePoint;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.data.FoeBotData;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.plays.defense.algorithms.interfaces.IDefensePointCalc;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.defense.DefenderRole;


/**
 * Deal with stop situation
 * 
 * @author Nicolai Ommer <nicolai.ommer@gmail.com>
 */
public class StopDefPointCalc extends ShortestPathDefensePointCalc
{
	private IDefensePointCalc	parent;
	
	
	/**
	 * @param parent
	 */
	public StopDefPointCalc(final IDefensePointCalc parent)
	{
		this.parent = parent;
	}
	
	
	@Override
	public Map<DefenderRole, DefensePoint> getDefenderDistribution(final MetisAiFrame frame,
			final List<DefenderRole> defenders,
			final List<FoeBotData> foeBotDataList)
	{
		Map<DefenderRole, DefensePoint> distribution = parent.getDefenderDistribution(frame, defenders, foeBotDataList);
		
		float specialPenAreaMargin = AIConfig.getGeometry().getBotRadius() + 10;
		for (Map.Entry<DefenderRole, DefensePoint> entry : distribution.entrySet())
		{
			DefensePoint defPoint = entry.getValue();
			IVector2 nearest2Goal = AIConfig
					.getGeometry()
					.getPenaltyAreaOur()
					.nearestPointOutside(AIConfig.getGeometry().getGoalOur().getGoalCenter(), defPoint, specialPenAreaMargin
					);
			IVector2 newDefpoint = nearest2Goal;
			float lineLength = GeoMath.distancePP(nearest2Goal, defPoint);
			final int resolution = 100;
			for (int i = 0; i < resolution; i++)
			{
				IVector2 p = GeoMath.stepAlongLine(defPoint, nearest2Goal, (lineLength / resolution) * i);
				if (isPointValid(frame, p, entry.getKey().getBotID()))
				{
					newDefpoint = p;
					break;
				}
			}
			distribution.put(entry.getKey(), new DefensePoint(newDefpoint, entry.getValue().getProtectAgainst()));
		}
		return distribution;
		
		// Map<DefenderRole, FoeBotData> distribution = super.getDefender2FoeDistribution(frame, defenders,
		// foeBotDataList);
		// Map<DefenderRole, DefensePoint> newDistribution = new HashMap<>();
		// for (Map.Entry<DefenderRole, FoeBotData> entry : distribution.entrySet())
		// {
		// FoeBotData fbd = entry.getValue();
		// float lineLength = GeoMath.distancePP(fbd.getNearestPointToOtherBot(), fbd.getPointNearestToOurGoal());
		// final int resolution = 100;
		// for (int i = 0; i < resolution; i++)
		// {
		// IVector2 p = GeoMath.stepAlongLine(fbd.getPointNearestToOurGoal(), fbd.getNearestPointToOtherBot(),
		// (lineLength / resolution) * i);
		// if (isPointValid(frame, p, entry.getKey().getBotID()))
		// {
		// newDistribution.put(entry.getKey(), new DefensePoint(p, entry.getValue().getBot()));
		// break;
		// }
		// }
		// }
		// return newDistribution;
	}
	
	
	/**
	 * TODO dirk, add comment!
	 * 
	 * @param currentFrame
	 * @param dp
	 * @param ownBotId
	 * @return
	 */
	public static boolean isPointValid(final MetisAiFrame currentFrame, final IVector2 dp, final BotID ownBotId)
	{
		Circle stopCircle = new Circle(currentFrame.getWorldFrame().getBall().getPos(), AIConfig.getGeometry()
				.getBotToBallDistanceStop() + (AIConfig.getGeometry().getBotRadius()));
		if (!AIConfig.getGeometry().getPenaltyAreaOur()
				.isPointInShape(dp, AIConfig.getGeometry().getBotRadius())
				&&
				AIConfig.getGeometry().getField().isPointInShape(dp) &&
				!stopCircle.isPointInShape(dp))
		{
			for (TrackedTigerBot bot : currentFrame.getWorldFrame().getTigerBotsVisible().values())
			{
				if (!ownBotId.equals(bot.getId()) && dp.equals(bot.getPos(), AIConfig.getGeometry().getBotRadius() * 2))
				{
					return false;
				}
			}
			return true;
		}
		return false;
	}
	
}
