/*
 * *********************************************************
 * Copyright (c) 2009 - 2015, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Apr 22, 2015
 * Author(s): FelixB <bayer.fel@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.plays.defense.algorithms.helpers;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.MetisAiFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.AIConfig;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.Geometry;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.data.DefensePoint;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.data.FoeBotData;


/**
 * Calculates a defense point which is as near to the covered bot as possible.
 * 
 * @author FelixB <bayer.fel@gmail.com>
 */
public class PassiveAgressivePointCalc implements IPointOnLine
{
	
	@Override
	public DefensePoint getPointOnLine(final DefensePoint defPoint, final MetisAiFrame frame)
	{
		FoeBotData foeBotData = defPoint.getFoeBotData();
		
		if (null == foeBotData)
		{
			return defPoint;
		}
		
		List<FoeBotData> passBots = new ArrayList<FoeBotData>();
		for (IVector2 passIntersection : foeBotData.getPassIntersections())
		{
			DefensePoint passIntersectionDefPoint = (DefensePoint) passIntersection;
			passBots.add(passIntersectionDefPoint.getFoeBotData());
		}
		Collections.sort(passBots, FoeBotData.DANGER_COMPARATOR);
		for (FoeBotData mostDangerous : passBots)
		{
			for (IVector2 passIntersection : foeBotData.getPassIntersections())
			{
				DefensePoint passIntersectionDefPoint = (DefensePoint) passIntersection;
				if (mostDangerous.equals(passIntersectionDefPoint.getFoeBotData()))
				{
					IVector2 validDefPoint = AIConfig
							.getGeometry()
							.getPenaltyAreaOur()
							.nearestPointOutside(passIntersectionDefPoint, frame.getWorldFrame().getBall().getPos(),
									Geometry.getPenaltyAreaMargin());
					
					validDefPoint = AIConfig.getGeometry().getField().nearestPointInside(validDefPoint);
					return new DefensePoint(validDefPoint, passIntersectionDefPoint.getFoeBotData());
				}
			}
		}
		return new DefensePoint(foeBotData.getNearestPointToOtherBot(), foeBotData);
	}
	
}
