/*
 * *********************************************************
 * Copyright (c) 2009 - 2014, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Nov 16, 2014
 * Author(s): Nicolai Ommer <nicolai.ommer@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.plays.others;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.AthenaAiFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.MetisAiFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.EGameState;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.ELetter;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.Vector2;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.AIConfig;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.plays.APlay;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.plays.EPlay;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.ARole;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.move.LetterRole;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.move.MoveRole;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.move.MoveRole.EMoveBehavior;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.sisyphus.driver.EMovingSpeed;


/**
 * Coordinate LetterRoles to form a word
 * 
 * @author Nicolai Ommer <nicolai.ommer@gmail.com>
 */
public class WordPlay extends APlay
{
	private static ELetter[]	word			= { ELetter.L, ELetter.I, ELetter.L, ELetter.L, ELetter.Y };
	
	private AthenaAiFrame		frame;
	private EGameState			gameState	= EGameState.UNKNOWN;
	
	
	/**
	 * 
	 */
	public WordPlay()
	{
		super(EPlay.LETTER);
	}
	
	
	@Override
	protected ARole onRemoveRole(MetisAiFrame frame)
	{
		return getLastRole();
	}
	
	
	@Override
	protected ARole onAddRole(MetisAiFrame frame)
	{
		return new MoveRole(EMoveBehavior.NORMAL);
	}
	
	
	@Override
	protected void onGameStateChanged(final EGameState gameState)
	{
		this.gameState = gameState;
		List<ARole> oldRoles = new ArrayList<ARole>(getRoles());
		List<List<IVector2>> absoluteWordPath = calculateAbsoluteWordPath();
		switch (gameState)
		{
			case STOPPED:
				for (int i = 0; i < Math.min(word.length, oldRoles.size()); i++)
				{
					MoveRole newRole = new MoveRole(absoluteWordPath.get(i).get(0), 0, EMovingSpeed.SLOW, 1);
					switchRoles(oldRoles.get(i), newRole);
				}
				break;
			case RUNNING:
				for (int i = 0; i < Math.min(word.length, oldRoles.size()); i++)
				{
					LetterRole newRole = new LetterRole(absoluteWordPath.get(i));
					switchRoles(oldRoles.get(i), newRole);
				}
				System.out.println("running");
				break;
			default:
				// ignore the rest
		}
	}
	
	
	private List<List<IVector2>> calculateAbsoluteWordPath()
	{
		Map<ELetter, List<IVector2>> letters = frame.getTacticalField().getLetters();
		Map<ELetter, Float> letterWidth = new HashMap<ELetter, Float>();
		
		for (Entry<ELetter, List<IVector2>> entry : letters.entrySet())
		{
			if (!entry.getValue().isEmpty())
			{
				IVector2 leftestPoint = entry.getValue().get(0);
				IVector2 rightestPoint = entry.getValue().get(0);
				for (IVector2 point : entry.getValue())
				{
					if (point.x() < leftestPoint.x())
					{
						leftestPoint = point;
					}
					if (point.x() > rightestPoint.x())
					{
						rightestPoint = point;
					}
				}
				letterWidth.put(entry.getKey(), rightestPoint.x() - leftestPoint.x());
			}
			else
			{
				letterWidth.put(entry.getKey(), 0f);
			}
		}
		
		float allWidth = 0;
		for (ELetter letter : word)
		{
			allWidth += letterWidth.get(letter);
		}
		allWidth += (word.length - 1) * AIConfig.getGeometry().getBotRadius() * 3;
		
		List<List<IVector2>> absolutePoints = new ArrayList<List<IVector2>>(word.length);
		
		float currentX = -(allWidth / 2);
		for (ELetter letter : word)
		{
			currentX += letterWidth.get(letter) / 2;
			List<IVector2> letterPoints = new ArrayList<IVector2>(letters.get(letter).size());
			IVector2 translation = new Vector2(currentX, 0);
			for (IVector2 point : letters.get(letter))
			{
				letterPoints.add(point.addNew(translation));
			}
			absolutePoints.add(letterPoints);
			currentX += (letterWidth.get(letter) / 2) + (AIConfig.getGeometry().getBotRadius() * 3);
		}
		
		return absolutePoints;
	}
	
	
	@Override
	protected void doUpdate(final AthenaAiFrame frame)
	{
		// called each frame after roles have been updated
		this.frame = frame;
		
		// TODO optional: use APlay#reorderRolesToDestinations to get shortest distances
		switch (gameState)
		{
			case STOPPED:
				
				break;
			case RUNNING:
				List<ARole> roles = getRoles();
				boolean readyFlag = true;
				for (int i = 0; i < Math.min(word.length, roles.size()); i++)
				{
					if (!(roles.get(i) instanceof LetterRole) || !((LetterRole) roles.get(i)).isReady())
					{
						readyFlag = false;
						break;
					}
				}
				if (readyFlag)
				{
					for (int i = 0; i < Math.min(word.length, roles.size()); i++)
					{
						((LetterRole) roles.get(i)).changeToLetter();
					}
				}
				
				break;
			default:
				// ignore the rest
		}
	}
	
	
	@Override
	public void updateBeforeRoles(final AthenaAiFrame frame)
	{
		// called each frame before roles will be updated
	}
	
}
