/*
 * *********************************************************
 * Copyright (c) 2009 - 2012, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: 27.11.2012
 * Author(s): Philipp
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.defense;

import java.io.Serializable;
import java.util.Comparator;
import java.util.List;

import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.ids.BotID;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.AIConfig;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.DefenseCalc;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.defense.data.DefensePoint;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.ARole;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.ERole;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.bots.EFeature;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.AMoveSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.DefenderSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.IMoveToSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.ISkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.statemachine.IRoleState;
import edu.dhbw.mannheim.tigers.sumatra.util.config.Configurable;


/**
 * DefenseRole for {@linkg DefensePoints}. Posionited a bot at a specififc position and look at a target (normally ball
 * position)
 * 
 * @author PhilippP
 */
public class DefenderRole extends ARole
{
	// --- analyzing specifications ---
	/** Point to protect against **/
	private DefensePoint												defPoint;
	
	@Configurable
	private static float												noPPAreaAroundPenaltyArea	= 2000;
	
	/**  */
	public static final Comparator<? super DefenderRole>	Y_COMPARATOR					= new YComparator();
	
	
	/**
	 */
	public DefenderRole()
	{
		super(ERole.DEFENDER);
		// new DefensePoint(AIConfig
		// .getGeometry()
		// .getGoalOur()
		// .getGoalCenter()
		// .addNew(
		// new Vector2(AIConfig.getGeometry().getDistanceToPenaltyArea() + AIConfig.getGeometry().getBotRadius()
		// + DefenseCalc.getPenaltyAreaMargin(), 0)));
		IRoleState normalDefend = new NormalDefendState();
		setInitialState(normalDefend);
		IRoleState moveState = new MoveFromField();
		addTransition(EStateId.OUTSIDE, EEvent.NEAR_DEF_POINT, new NormalDefendState());
		addTransition(EStateId.NORMAL, EEvent.FAR_FROM_DEF_POINT, moveState);
	}
	
	private enum EStateId
	{
		NORMAL,
		OUTSIDE,
	}
	
	private enum EEvent
	{
		DONE,
		NEAR_DEF_POINT,
		FAR_FROM_DEF_POINT,
	}
	
	
	// --------------------------------------------------------------------------
	// --- getter/setter --------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * Set the Defender Position
	 * 
	 * @param defPoint - Position to Defend
	 */
	public void setDefPoint(final DefensePoint defPoint)
	{
		assert defPoint != null : "Defpoint should not be null!";
		if (defPoint != null)
		{
			this.defPoint = defPoint;
		}
	}
	
	
	/**
	 * Returns the acutal defensPoint
	 * 
	 * @return DefensPoints
	 */
	public IVector2 getDefPoint()
	{
		
		return defPoint;
	}
	
	
	@Override
	public void fillNeededFeatures(final List<EFeature> features)
	{
		features.add(EFeature.MOVE);
	}
	
	
	// --------------------------------------------------------------------------
	// --- InnerClasses --------------------------------------------------------
	// --------------------------------------------------------------------------
	private static class YComparator implements Comparator<DefenderRole>, Serializable
	{
		
		/**  */
		private static final long	serialVersionUID	= 1794858044291002364L;
		
		
		@Override
		public int compare(final DefenderRole v1, final DefenderRole v2)
		{
			if (v1.getPos().y() > v2.getPos().y())
			{
				return 1;
			} else if (v1.getPos().y() < v2.getPos().y())
			{
				return -1;
			} else
			{
				return 0;
			}
		}
	}
	
	private class MoveFromField implements IRoleState
	{
		private IMoveToSkill	skill;
		
		
		@Override
		public void onSkillStarted(final ISkill skill, final BotID botID)
		{
		}
		
		
		@Override
		public void onSkillCompleted(final ISkill skill, final BotID botID)
		{
		}
		
		
		@Override
		public void doEntryActions()
		{
			skill = AMoveSkill.createMoveToSkill();
			skill.getMoveCon().updateDestination(defPoint);
			setNewSkill(skill);
		}
		
		
		@Override
		public void doExitActions()
		{
		}
		
		
		@Override
		public void doUpdate()
		{
			skill.getMoveCon().updateDestination(defPoint);
			if (AIConfig.getGeometry().getPenaltyAreaOur().isPointInShape(getPos(), DefenseCalc.getPenaltyAreaMargin() +
					noPPAreaAroundPenaltyArea))
			{
				nextState(EEvent.NEAR_DEF_POINT);
			}
		}
		
		
		@Override
		public Enum<? extends Enum<?>> getIdentifier()
		{
			return EStateId.OUTSIDE;
		}
		
	}
	
	private class NormalDefendState implements IRoleState
	{
		private DefenderSkill	defSkill	= null;
		
		
		@Override
		public void onSkillStarted(final ISkill skill, final BotID botID)
		{
			
		}
		
		
		@Override
		public void onSkillCompleted(final ISkill skill, final BotID botID)
		{
			
		}
		
		
		@Override
		public void doEntryActions()
		{
			defSkill = new DefenderSkill();
			setNewSkill(defSkill);
		}
		
		
		@Override
		public void doExitActions()
		{
		}
		
		
		@Override
		public void doUpdate()
		{
			if (!AIConfig.getGeometry().getPenaltyAreaOur().isPointInShape(getPos(), DefenseCalc.getPenaltyAreaMargin() +
					noPPAreaAroundPenaltyArea))
			{
				nextState(EEvent.FAR_FROM_DEF_POINT);
			}
			
			defSkill.getMoveCon().setBotsObstacle(false);
			defSkill.getMoveCon().setDriveFast(true);
			if (defPoint != null)
			{
				defSkill.setDefensePoint(defPoint);
			}
		}
		
		
		@Override
		public Enum<? extends Enum<?>> getIdentifier()
		{
			return EStateId.NORMAL;
		}
	}
	
}
