/*
 * *********************************************************
 * Copyright (c) 2009 - 2014, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Nov 16, 2014
 * Author(s): Nicolai Ommer <nicolai.ommer@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.move;

import java.util.List;

import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.ids.BotID;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.ARole;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.ERole;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.bots.EFeature;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.AMoveSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.IMoveToSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.ISkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.LetterSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.statemachine.IRoleState;


/**
 * Drive a letter
 * 
 * @author Nicolai Ommer <nicolai.ommer@gmail.com>
 */
public class LetterRole extends ARole
{
	private final List<IVector2>	pathpoints;
	
	
	/**
	 * @param pathpoints
	 */
	public LetterRole(final List<IVector2> pathpoints)
	{
		super(ERole.LETTER);
		this.pathpoints = pathpoints;
		IRoleState initState = new InitState();
		IRoleState letterState = new LetterState();
		setInitialState(initState);
		addTransition(null, EEvent.STOPPED, initState);
		addTransition(null, EEvent.RUNNING, letterState);
		addTransition(null, EEvent.WAIT, new WaitState());
	}
	
	// --------------------------------------------------------------------------
	// --- states ---------------------------------------------------------------
	// --------------------------------------------------------------------------
	
	private enum EStateId
	{
		INIT,
		LETTER,
		WAITING
	}
	
	private enum EEvent
	{
		STOPPED,
		RUNNING,
		WAIT
	}
	
	
	@Override
	public void fillNeededFeatures(final List<EFeature> features)
	{
		// no features needed
	}
	
	
	/**
	 */
	public void changeToInit()
	{
		nextState(EEvent.STOPPED);
	}
	
	
	/**
	 * Has the role reached its initial position on the letter?
	 * 
	 * @return
	 */
	public boolean isReady()
	{
		
		switch ((EStateId) getCurrentState())
		{
			case WAITING:
				return true;
			case INIT:
			case LETTER:
			default:
				return false;
				
		}
		
		
	}
	
	
	/**
	 */
	public void changeToLetter()
	{
		nextState(EEvent.RUNNING);
	}
	
	private class InitState implements IRoleState
	{
		
		@Override
		public void onSkillStarted(final ISkill skill, final BotID botID)
		{
			
		}
		
		
		@Override
		public void onSkillCompleted(final ISkill skill, final BotID botID)
		{
			nextState(EEvent.WAIT);
		}
		
		
		@Override
		public void doEntryActions()
		{
			IMoveToSkill moveSkill = AMoveSkill.createMoveToSkill();
			moveSkill.setDoComplete(true);
			if (!pathpoints.isEmpty())
			{
				moveSkill.getMoveCon().updateDestination(pathpoints.get(0));
			}
			setNewSkill(moveSkill);
			
		}
		
		
		@Override
		public void doExitActions()
		{
		}
		
		
		@Override
		public void doUpdate()
		{
		}
		
		
		@Override
		public Enum<? extends Enum<?>> getIdentifier()
		{
			return EStateId.INIT;
		}
		
	}
	
	private static class WaitState implements IRoleState
	{
		
		@Override
		public void onSkillStarted(final ISkill skill, final BotID botID)
		{
		}
		
		
		@Override
		public void onSkillCompleted(final ISkill skill, final BotID botID)
		{
			
		}
		
		
		@Override
		public void doEntryActions()
		{
		}
		
		
		@Override
		public void doExitActions()
		{
		}
		
		
		@Override
		public void doUpdate()
		{
		}
		
		
		@Override
		public Enum<? extends Enum<?>> getIdentifier()
		{
			return EStateId.WAITING;
		}
	}
	
	private class LetterState implements IRoleState
	{
		
		@Override
		public void doEntryActions()
		{
			setNewSkill(new LetterSkill(pathpoints));
		}
		
		
		@Override
		public void doUpdate()
		{
		}
		
		
		@Override
		public void doExitActions()
		{
		}
		
		
		@Override
		public void onSkillStarted(final ISkill skill, final BotID botID)
		{
		}
		
		
		@Override
		public void onSkillCompleted(final ISkill skill, final BotID botID)
		{
		}
		
		
		@Override
		public Enum<? extends Enum<?>> getIdentifier()
		{
			return EStateId.LETTER;
		}
	}
}
