/*
 * *********************************************************
 * Copyright (c) 2009 - 2014, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Oct 30, 2014
 * Author(s): Nicolai Ommer <nicolai.ommer@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.sisyphus.driver;

import java.util.List;

import edu.dhbw.mannheim.tigers.sumatra.model.data.math.AiMath;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.IDrawableShape;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector3;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.Vector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.Vector3;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.TrackedTigerBot;


/**
 * A path driver gives you your next destination, given a path or spline
 * 
 * @author Nicolai Ommer <nicolai.ommer@gmail.com>
 */
public interface IPathDriver
{
	
	/**
	 * Get the next point+orientation on the path, given our current position and velocity
	 * 
	 * @param bot
	 * @return [mm,mm,rad]
	 */
	IVector3 getNextDestination(TrackedTigerBot bot);
	
	
	/**
	 * Get next/current velocity (x,y,w) for the robot
	 * 
	 * @param bot
	 * @return [m/s,m/s,rad/s]
	 */
	IVector3 getNextVelocity(TrackedTigerBot bot);
	
	
	/**
	 * @param bot
	 * @param dt
	 * @return
	 */
	default IVector3 getNextLocalVelocity(final TrackedTigerBot bot, final float dt)
	{
		IVector3 vel = getNextVelocity(bot);
		Vector2 localVel = AiMath.convertGlobalBotVector2Local(vel.getXYVector(), bot.getAngle());
		
		float velComp = -vel.z() * dt;
		localVel.turn(velComp);
		return new Vector3(localVel.x(), localVel.y(), vel.z());
	}
	
	
	/**
	 * @param bot
	 * @return [mm]
	 */
	IVector3 getFinalDestination(TrackedTigerBot bot);
	
	
	/**
	 * The moving speed should be considered, if not done on a higher level
	 * 
	 * @param speed
	 */
	void setMovingSpeed(EMovingSpeed speed);
	
	
	/**
	 * @param bot
	 * @return
	 */
	float getRemainingTime(final TrackedTigerBot bot);
	
	
	/**
	 * Add shapes to the list, if you like
	 * 
	 * @param bot
	 * @param shapes
	 */
	void decoratePath(TrackedTigerBot bot, final List<IDrawableShape> shapes);
	
	
	/**
	 * Add shapes to the list, if you like
	 * 
	 * @param bot
	 * @param shapes
	 */
	void decoratePathDebug(TrackedTigerBot bot, final List<IDrawableShape> shapes);
}
