/*
 * *********************************************************
 * Copyright (c) 2009 - 2015, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: May 13, 2015
 * Author(s): Nicolai Ommer <nicolai.ommer@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.sisyphus.driver.skills;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sumatra.model.data.DynamicPosition;
import edu.dhbw.mannheim.tigers.sumatra.model.data.frames.WorldFrame;
import edu.dhbw.mannheim.tigers.sumatra.model.data.math.GeoMath;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.DrawablePoint;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.IDrawableShape;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.triangle.DrawableTriangle;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.triangle.Triangle;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.Vector2;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.line.DrawableLine;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.line.Line;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.TrackedTigerBot;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.sisyphus.driver.PositionDriver;
import edu.dhbw.mannheim.tigers.sumatra.util.config.Configurable;


/**
 * Kick the ball. This driver assumes that the bot is already behind the ball!
 * 
 * @author Nicolai Ommer <nicolai.ommer@gmail.com>
 */
public class KickBallDriver extends PositionDriver implements ISkillDriver
{
	@SuppressWarnings("unused")
	private static final Logger	log							= Logger.getLogger(KickBallDriver.class.getName());
	
	@Configurable(comment = "Dist [mm] that the destination is set behind ball hit point")
	private static int				distBehindBallHitTarget	= 50;
	
	@Configurable(comment = "Angle [rad] of the triangle")
	private static float				triangleAngleOuter		= 0.7f;
	
	private List<IDrawableShape>	shapes						= new ArrayList<>();
	private final DynamicPosition	receiver;
	private boolean					done							= false;
	
	
	/**
	 * @param receiver
	 */
	public KickBallDriver(final DynamicPosition receiver)
	{
		this.receiver = receiver;
	}
	
	
	/**
	 * @param bot
	 * @param wFrame
	 */
	@Override
	public void update(final TrackedTigerBot bot, final WorldFrame wFrame)
	{
		final List<IDrawableShape> shapes = new ArrayList<>();
		final IVector2 ballPos = wFrame.getBall().getPos();
		final IVector2 ballVel = wFrame.getBall().getVel();
		
		IVector2 dir = receiver.subtractNew(ballPos);
		Triangle triangleOuter = new Triangle(
				ballPos.addNew(dir.scaleToNew(50)),
				ballPos.addNew(dir.scaleToNew(-50000).turnNew(triangleAngleOuter)),
				ballPos.addNew(dir.scaleToNew(-50000).turnNew(-triangleAngleOuter)));
		
		if (!triangleOuter.isPointInShape(bot.getPos()))
		{
			done = true;
		}
		
		final IVector2 destination;
		IVector2 futureBallPos = ballPos; // TODO
		IVector2 direction = receiver.subtractNew(ballPos);
		float secDist2Ball = 200;
		if (GeoMath.distancePP(bot.getPos(), futureBallPos) > 200)
		{
			destination = futureBallPos.addNew(dir
					.scaleToNew(-(bot.getBot().getCenter2DribblerDist() - distBehindBallHitTarget)));
		} else
		{
			IVector2 lp = GeoMath.leadPointOnLine(bot.getPos(), futureBallPos, receiver);
			if ((GeoMath.distancePP(lp, bot.getPos()) < 30) || (bot.getVel().getLength2() < 0.2f))
			{
				IVector2 botDestForHit = futureBallPos.addNew(direction
						.scaleToNew(-bot.getBot().getCenter2DribblerDist()));
				IVector2 dest = GeoMath.stepAlongLine(botDestForHit, bot.getPos(), -distBehindBallHitTarget);
				destination = GeoMath.stepAlongLine(bot.getPos(), dest,
						Math.min(300, GeoMath.distancePP(dest, bot.getPos())));
			} else
			{
				IVector2 dest = lp.addNew(direction.scaleToNew(100));
				if (GeoMath.distancePP(dest, ballPos) < secDist2Ball)
				{
					destination = GeoMath.stepAlongLine(ballPos, dest, secDist2Ball);
				} else
				{
					destination = dest;
				}
			}
		}
		// if (GeoMath.distancePP(bot.getPos(), futureBallPos) <= getBot().getCenter2DribblerDist())
		// {
		// // future ball is inside bot. Normal calculation will not work properly,
		// // so just send the bot with its dribbler onto the ball.
		// destination = futureBallPos.addNew(direction.scaleToNew(-getBot().getCenter2DribblerDist()));
		// } else
		// {
		// IVector2 botDestForHit = futureBallPos.addNew(direction
		// .scaleToNew(-getBot().getCenter2DribblerDist()));
		// destination = GeoMath.stepAlongLine(botDestForHit, bot.getPos(), -distBehindBallHitTarget);
		// }
		setDestination(destination);
		setOrientation(direction.getAngle());
		
		
		if (ballVel.getLength2() > 0.1f)
		{
			shapes.add(new DrawableLine(new Line(ballPos, ballVel.scaleToNew(1000)), Color.blue));
		}
		shapes.add(new DrawableLine(new Line(ballPos, receiver.subtractNew(ballPos).scaleToNew(5000)), Color.cyan));
		shapes.add(new DrawableLine(new Line(bot.getPos(), dir.scaleToNew(1000)), Color.orange));
		shapes.add(new DrawableLine(new Line(bot.getPos(), receiver.subtractNew(ballPos).scaleToNew(1000)), Color.magenta));
		shapes.add(new DrawableLine(new Line(bot.getPos(), new Vector2(bot.getAngle()).scaleTo(5000)), Color.BLACK));
		
		DrawablePoint dpState = new DrawablePoint(bot.getPos().addNew(new Vector2(-150, 100)));
		dpState.setText("KickBall");
		shapes.add(dpState);
		
		DrawableTriangle dTriangleOuter = new DrawableTriangle(
				ballPos.addNew(dir.scaleToNew(50)),
				ballPos.addNew(dir.scaleToNew(-1000).turnNew(triangleAngleOuter)),
				ballPos.addNew(dir.scaleToNew(-1000).turnNew(-triangleAngleOuter)),
				new Color(0f, 1f, 1f, 0.2f));
		dTriangleOuter.setFill(true);
		shapes.add(dTriangleOuter);
		this.shapes = shapes;
	}
	
	
	@Override
	public void decoratePathDebug(final TrackedTigerBot bot, final List<IDrawableShape> shapes)
	{
		super.decoratePathDebug(bot, shapes);
		shapes.addAll(this.shapes);
	}
	
	
	/**
	 * @return the done
	 */
	@Override
	public final boolean isDone()
	{
		return done;
	}
	
	
	@Override
	public ESkillDriver getType()
	{
		return ESkillDriver.KICK_BALL;
	}
}
