/*
 * *********************************************************
 * Copyright (c) 2009 - 2015, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Feb 28, 2015
 * Author(s): Nicolai Ommer <nicolai.ommer@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.bots;

import org.apache.commons.configuration.HierarchicalConfiguration;
import org.apache.commons.configuration.SubnodeConfiguration;

import com.sleepycat.persist.model.Persistent;

import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.ids.BotID;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.bots.communication.ENetworkState;


/**
 * @author Nicolai Ommer <nicolai.ommer@gmail.com>
 */
@Persistent
public abstract class SimBot extends ABot
{
	private ENetworkState	networkState	= ENetworkState.ONLINE;
	
	
	protected SimBot()
	{
		super();
	}
	
	
	/**
	 * @param botConfig
	 */
	public SimBot(final SubnodeConfiguration botConfig)
	{
		super(botConfig);
		networkState = ENetworkState.valueOf(botConfig.getString("networkState", "OFFLINE"));
	}
	
	
	/**
	 * @param type
	 * @param id
	 * @param baseStationKey
	 * @param mcastDelegateKey
	 */
	public SimBot(final EBotType type, final BotID id, final int baseStationKey, final int mcastDelegateKey)
	{
		super(type, id, baseStationKey, mcastDelegateKey);
		networkState = ENetworkState.OFFLINE;
	}
	
	
	@Override
	public HierarchicalConfiguration getConfiguration()
	{
		HierarchicalConfiguration config = super.getConfiguration();
		config.addProperty("bot.networkState", networkState);
		return config;
	}
	
	
	/**
	 * @param networkState the networkState to set
	 */
	public void setNetworkState(final ENetworkState networkState)
	{
		this.networkState = networkState;
		notifyNetworkStateChanged(networkState);
	}
	
	
	@Override
	public ENetworkState getNetworkState()
	{
		return networkState;
	}
}
