/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: 07.08.2010
 * Author(s):
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem;

import java.util.List;

import edu.dhbw.mannheim.tigers.sumatra.model.data.DynamicPosition;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.ASkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.BlockSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.ChipFastSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.ChipFastTestSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.CurveTestSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.DefenderSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.EightSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.IdleSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.InterceptionSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.KickSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.KickSkill.EKickMode;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.KickSkill.EMoveMode;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.KickTestSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.LatencyTestSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.LetterSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.LookAtBallTestSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.MoveBallToSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.MoveTestSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.MoveToSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.MovingKickSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.PenaltyKeeperSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.PenaltyShootSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.PenaltyShootSkill.ERotateDirection;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.PolicyTrainerSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.PositionSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.PullBackSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.ReceiverSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.RedirectSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.RotateTestSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.StraightMoveSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.TurnAroundBallPosSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.TurnAroundBallSplineSkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.TurnWithBallSkill;
import edu.dhbw.mannheim.tigers.sumatra.util.IInstanceableEnum;
import edu.dhbw.mannheim.tigers.sumatra.util.InstanceableClass;
import edu.dhbw.mannheim.tigers.sumatra.util.InstanceableParameter;


/**
 * An enumeration that uniquely identifies each {@link ASkill}-implementation
 * 
 * @author Gero
 * @author Nicolai Ommer <nicolai.ommer@gmail.com>
 */
public enum ESkillName implements IInstanceableEnum
{
	
	/**  */
	POSITION(new InstanceableClass(PositionSkill.class, new InstanceableParameter(IVector2.class, "destination", "0,0"),
			new InstanceableParameter(Float.TYPE, "orientation", "0"))),
	
	/**  */
	KICK(new InstanceableClass(KickSkill.class, new InstanceableParameter(DynamicPosition.class, "receiver", "4050,0"),
			new InstanceableParameter(EKickMode.class, "kickMode", "MAX"), new InstanceableParameter(EMoveMode.class,
					"modeMode", "NORMAL"))),
	/**  */
	REDIRECT(new InstanceableClass(RedirectSkill.class, new InstanceableParameter(DynamicPosition.class, "target",
			"3205,0"))),
	
	/**  */
	KICK_TEST(new InstanceableClass(KickTestSkill.class, new InstanceableParameter(DynamicPosition.class, "target",
			"3025,0"),
			new InstanceableParameter(Integer.TYPE, "duration", "10000"))),
	
	/**  */
	CHIP_FAST(new InstanceableClass(ChipFastSkill.class, new InstanceableParameter(DynamicPosition.class, "target",
			"3025,0"))),
	
	/**  */
	CHIP_FAST_TEST(new InstanceableClass(ChipFastTestSkill.class,
			new InstanceableParameter(DynamicPosition.class, "target", "0,0"),
			new InstanceableParameter(Integer.TYPE, "duration", "5000"),
			new InstanceableParameter(Integer.TYPE, "dribble", "0"))),
	
	/** */
	STRAIGHT_MOVE(new InstanceableClass(StraightMoveSkill.class, new InstanceableParameter(Integer.TYPE, "Distance",
			"1000"), new InstanceableParameter(Float.TYPE, "Angle", "0.0"))),
	/**  */
	TURN_AROUND_BALL_SPLINE(new InstanceableClass(TurnAroundBallSplineSkill.class, new InstanceableParameter(
			IVector2.class, "lookAtTarget", "0, 0"))),
	/**  */
	TURN_AROUND_BALL_SPLINE_ANGLE(new InstanceableClass(TurnAroundBallSplineSkill.class, new InstanceableParameter(
			Float.TYPE, "angle", "1.5708"))),
	/**  */
	TURN_AROUND_BALL_POS(new InstanceableClass(TurnAroundBallPosSkill.class, new InstanceableParameter(Float.TYPE,
			"radius", "50"))),
	/**  */
	TURN_WITH_BALL(new InstanceableClass(TurnWithBallSkill.class, new InstanceableParameter(IVector2.class,
			"lookAtTarget", "0,0"))),
	
	/**  */
	TURN(new InstanceableClass(CurveTestSkill.class, new InstanceableParameter(Float.TYPE, "size", "100"),
			new InstanceableParameter(Float.TYPE, "angle", "3.14"))),
	/**  */
	ROTATE(new InstanceableClass(RotateTestSkill.class, new InstanceableParameter(Float.TYPE, "angle", "1.5708"))),
	/**  */
	EIGHT(new InstanceableClass(EightSkill.class, new InstanceableParameter(Float.TYPE, "size", "1500"))),
	/**  */
	LOOK_AT_BALL(new InstanceableClass(LookAtBallTestSkill.class)),
	
	/**  */
	DEFENDER(new InstanceableClass(DefenderSkill.class)),
	/**  */
	BLOCK(new InstanceableClass(BlockSkill.class)),
	/** */
	INTERCEPTION(new InstanceableClass(InterceptionSkill.class)),
	/**  */
	RECEIVER(new InstanceableClass(ReceiverSkill.class)),
	
	/**  */
	PENALTY_SHOOT(new InstanceableClass(PenaltyShootSkill.class, new InstanceableParameter(ERotateDirection.class,
			"rotateDirection",
			"LEFT"))),
	/**  */
	PENALTY_KEEPER(new InstanceableClass(PenaltyKeeperSkill.class, new InstanceableParameter(DynamicPosition.class,
			"shooterID",
			"3,YELLOW"))),
	
	// ************************
	// *** utility skills
	// ************************
	
	/**  */
	MOVE_BALL_TO(new InstanceableClass(MoveBallToSkill.class,
			new InstanceableParameter(IVector2.class, "Target", "0, 0"))),
	/**  */
	LATENCY_TEST(new InstanceableClass(LatencyTestSkill.class)),
	/**  */
	POLICY_TRAINER(new InstanceableClass(PolicyTrainerSkill.class)),
	/**  */
	LETTER(new InstanceableClass(LetterSkill.class, new InstanceableParameter(List.class, "pathPoints",
			"500,1000;500,0;0,0", IVector2.class))),
	/**  */
	MOVE_TEST(new InstanceableClass(MoveTestSkill.class, new InstanceableParameter(MoveTestSkill.EMode.class, "mode",
			"FWD"), new InstanceableParameter(Boolean.TYPE, "loop", "false"))),
	
	// ************************
	// *** normal skills
	// ************************
	/** */
	IDLE(new InstanceableClass(IdleSkill.class)),
	/**  */
	MOVE_TO(new InstanceableClass(MoveToSkill.class)),
	
	// ************************
	// *** to be checked
	// ************************
	
	/**  */
	PULL_BACK(new InstanceableClass(PullBackSkill.class,
			new InstanceableParameter(IVector2.class, "endOrientation", "1,0"),
			new InstanceableParameter(Float.TYPE, "pullDistance", "500"),
			new InstanceableParameter(IVector2.class, "pullDirection", "-1,0"),
			new InstanceableParameter(Float.TYPE, "speed", "1"))),
	
	/**  */
	MOVING_KICK(new InstanceableClass(MovingKickSkill.class,
			new InstanceableParameter(IVector2.class, "target", "4000,0"))), ;
	
	private final InstanceableClass	clazz;
	
	
	/**
	 */
	private ESkillName(final InstanceableClass clazz)
	{
		this.clazz = clazz;
	}
	
	
	/**
	 * @return the paramImpls
	 */
	@Override
	public final InstanceableClass getInstanceableClass()
	{
		return clazz;
	}
}
