/*
 * *********************************************************
 * Copyright (c) 2009 - 2014, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: May 6, 2014
 * Author(s): Nicolai Ommer <nicolai.ommer@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills;

import java.util.List;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sumatra.model.data.DynamicPosition;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.ACommand;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.ESkillName;
import edu.dhbw.mannheim.tigers.sumatra.util.csvexporter.BallWatcher;
import edu.dhbw.mannheim.tigers.sumatra.util.csvexporter.CSVExporter;
import edu.dhbw.mannheim.tigers.sumatra.util.csvexporter.IBallWatcherObserver;


/**
 * Use custom duration for kicking
 * 
 * @author Nicolai Ommer <nicolai.ommer@gmail.com>
 */
public class KickTestSkill extends KickSkill implements IBallWatcherObserver
{
	@SuppressWarnings("unused")
	private static final Logger	log	= Logger.getLogger(KickTestSkill.class.getName());
	private final int					duration;
	private final BallWatcher		watcher;
	
	
	/**
	 * @param target
	 * @param duration
	 */
	public KickTestSkill(final DynamicPosition target, final int duration)
	{
		super(ESkillName.KICK_TEST, target, EKickMode.PASS, EMoveMode.NORMAL);
		this.duration = duration;
		watcher = new BallWatcher("ballKick/ballKick" + System.currentTimeMillis());
		watcher.addObserver(this);
	}
	
	
	@Override
	protected int calcStraightDuration(final float length)
	{
		return duration;
	}
	
	
	@Override
	public void doCalcEntryActions(final List<ACommand> cmds)
	{
		super.doCalcEntryActions(cmds);
		watcher.start();
	}
	
	
	@Override
	public void beforeExport(final CSVExporter exporter)
	{
		exporter.addValues(duration);
	}
	
	
	@Override
	public void onAddCustomData(final List<Object> data)
	{
	}
	
	
	@Override
	public void postProcessing(final String fileName)
	{
		// MatlabProxy mp;
		// try
		// {
		// mp = MatlabConnection.getMatlabProxy();
		// float goalX = (AIConfig.getGeometry().getFieldLength() / 2) + AIConfig.getGeometry().getGoalDepth();
		// Object[] res = mp.returningFeval("kickDistApprox", 4, fileName, goalX / 1000);
		// double[] kickDists = (double[]) res[0];
		// double[] endVel = (double[]) res[1];
		// double[] avgVel = (double[]) res[2];
		// double[] initVel = (double[]) res[3];
		// log.info("duration: " + duration + " kickDist: " + kickDists[0] + " endVel: " + endVel[0] + " avgVel: "
		// + avgVel[0] + " initVel: "
		// + initVel[0]);
		//
		// } catch (MatlabConnectionException err)
		// {
		// // ignore
		// } catch (MatlabInvocationException err)
		// {
		// log.error("Error evaluating matlab function: " + err.getMessage(), err);
		// }
	}
}
