/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: 07.08.2010
 * Author(s):
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.types;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.moduli.AModule;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.AresData;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.ETeamColor;
import edu.dhbw.mannheim.tigers.sumatra.model.data.trackedobjects.ids.BotID;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.pandora.roles.ARole;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.sisyphus.PathFinderThread;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.skillsystem.skills.ISkill;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.observer.ISkillSystemObserver;
import edu.dhbw.mannheim.tigers.sumatra.util.GeneralPurposeTimer;


/**
 * The base class for every implementation of a skill system
 * 
 * @author Gero
 */
public abstract class ASkillSystem extends AModule implements IEmergencyStop
{
	private static final Logger					log			= Logger.getLogger(ASkillSystem.class.getName());
	
	/** */
	public static final String						MODULE_TYPE	= "AMoveSystem";
	/** */
	public static final String						MODULE_ID	= "skillsystem";
	
	private final Set<ISkillSystemObserver>	observers	= Collections
																					.newSetFromMap(new ConcurrentHashMap<ISkillSystemObserver, Boolean>());
	
	
	/**
	 * @param botId
	 * @param skill
	 */
	public abstract void execute(BotID botId, ISkill skill);
	
	
	/**
	 * @param botId
	 */
	public abstract void reset(final BotID botId);
	
	
	// --------------------------------------------------------------------------
	// --- getter/setter --------------------------------------------------------
	// --------------------------------------------------------------------------
	
	
	@Override
	public void deinitModule()
	{
		observers.clear();
	}
	
	
	/**
	 * Reset skill system aka delete roles from given ai color.
	 * Used after crash in AI
	 * 
	 * @param color
	 */
	public void reset(final ETeamColor color)
	{
		List<ISkillSystemObserver> copy = new ArrayList<ISkillSystemObserver>(observers);
		for (ISkillSystemObserver o : copy)
		{
			if (o instanceof ARole)
			{
				ARole role = (ARole) o;
				if (role.getBotID().getTeamColor() == color)
				{
					observers.remove(o);
				}
			}
		}
	}
	
	
	/**
	 */
	protected void checkObserversCleared()
	{
		// delay the check, so that the skillexecuter has time to complete its skills
		GeneralPurposeTimer.getInstance().schedule(new TimerTask()
		{
			@Override
			public void run()
			{
				if (!observers.isEmpty())
				{
					log.warn("There are still observers left: " + observers);
				}
				observers.clear();
			}
		}, 1000);
	}
	
	
	/**
	 * @param observer
	 */
	public void addObserver(final ISkillSystemObserver observer)
	{
		synchronized (observers)
		{
			observers.add(observer);
		}
	}
	
	
	/**
	 * @param observer
	 */
	public void removeObserver(final ISkillSystemObserver observer)
	{
		synchronized (observers)
		{
			observers.remove(observer);
		}
	}
	
	
	protected void notifySkillStarted(final ISkill skill, final BotID botId)
	{
		synchronized (observers)
		{
			for (final ISkillSystemObserver observer : observers)
			{
				observer.onSkillStarted(skill, botId);
			}
		}
	}
	
	
	protected void notifySkillCompleted(final ISkill skill, final BotID botId)
	{
		synchronized (observers)
		{
			for (final ISkillSystemObserver observer : observers)
			{
				observer.onSkillCompleted(skill, botId);
			}
		}
	}
	
	
	/**
	 * @return
	 */
	public abstract AresData getLatestAresData();
	
	
	/**
	 * @return
	 */
	public abstract PathFinderThread getPathFinderScheduler();
}
