/*
 * *********************************************************
 * Copyright (c) 2009 - 2015, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Apr 19, 2015
 * Author(s): Nicolai Ommer <nicolai.ommer@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.worldpredictor.oextkal;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.log4j.Level;
import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;

import edu.dhbw.mannheim.tigers.sumatra.model.data.math.GeoMath;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.cam.CamBall;
import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.cam.CamRobot;
import edu.dhbw.mannheim.tigers.sumatra.model.data.shapes.vector.IVector2;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.config.Geometry;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.worldpredictor.kalman.oextkal.BallCorrector;
import edu.dhbw.mannheim.tigers.sumatra.util.SumatraSetupHelper;
import edu.dhbw.mannheim.tigers.sumatra.util.clock.StaticSimulationClock;
import edu.dhbw.mannheim.tigers.sumatra.util.clock.SumatraClock;
import edu.dhbw.mannheim.tigers.sumatra.util.csvexporter.CSVExporter;


/**
 * @author Nicolai Ommer <nicolai.ommer@gmail.com>
 */
public class BallCorrectorTest
{
	private static final String	CHIP_KICK_FILE	= "moduli_2015/ballChip/ballChip1429268018444";
	private static final String	POSTFIX			= "_corr";
	
	static
	{
		SumatraSetupHelper.setupSumatra();
		SumatraSetupHelper.changeLogLevel(Level.TRACE);
	}
	
	
	/**
	 */
	@Test
	@Ignore
	public void testRecordedChip()
	{
		List<CamBall> balls;
		List<CamRobot> robots;
		try
		{
			balls = Files.lines(Paths.get("logs/" + CHIP_KICK_FILE + ".csv"))
					.filter(line -> !line.startsWith("#"))
					.map(line -> line.split(","))
					.filter(arr -> arr.length > 4)
					.map(arr -> new CamBall(0, 0, Float.valueOf(arr[3]), Float.valueOf(arr[4]), 0, 0, 0, Long
							.valueOf(arr[1]), Integer.valueOf(arr[2])))
					.collect(Collectors.toList());
			robots = Files.lines(Paths.get("logs/" + CHIP_KICK_FILE + ".csv"))
					.filter(line -> !line.startsWith("#"))
					.map(line -> line.split(","))
					.filter(arr -> arr.length > 13)
					.map(arr -> new CamRobot(0, 0, Float.valueOf(arr[11]), Float.valueOf(arr[12]), Float.valueOf(arr[13]),
							0, 0, 0, Long
									.valueOf(arr[1]), Integer.valueOf(arr[2])))
					.collect(Collectors.toList());
		} catch (IOException err)
		{
			Assert.fail("File not found!");
			throw new IllegalStateException();
		}
		
		if (robots.size() != balls.size())
		{
			// if no robot information available, fake it :P
			robots.clear();
			IVector2 botPos = GeoMath.stepAlongLine(balls.get(0).getPos().getXYVector(), balls.get(balls.size() - 1)
					.getPos()
					.getXYVector(), -(Geometry.getCenter2DribblerDistDefault() + 10));
			float orientation = balls.get(balls.size() - 1).getPos().getXYVector()
					.subtractNew(balls.get(0).getPos().getXYVector()).getAngle();
			CamRobot robot = new CamRobot(0, 0, botPos.x(), botPos.y(), orientation, 0, 0, 0, 0, 0);
			for (int i = 0; i < balls.size(); i++)
			{
				robots.add(robot);
			}
		}
		
		StaticSimulationClock clock = new StaticSimulationClock();
		clock.setNanoTime(balls.get(0).getTimestamp());
		SumatraClock.setClock(clock);
		
		BallCorrector ballCorrector = new BallCorrector();
		CSVExporter exporter = new CSVExporter(CHIP_KICK_FILE + POSTFIX, false);
		
		for (int i = 0; i < balls.size(); i++)
		{
			CamBall ball = balls.get(i);
			CamRobot robot = robots.get(i);
			List<CamBall> frameBalls = new ArrayList<>(1);
			frameBalls.add(ball);
			List<CamRobot> robotsYellow = new ArrayList<>(1);
			List<CamRobot> robotsBlue = new ArrayList<>(0);
			robotsYellow.add(robot);
			
			CamBall corrBall = ballCorrector.correctBall(frameBalls, robotsYellow, robotsBlue);
			
			// System.out.println(String.format("%d,%d,%f,%f,%f,%f", ball.getTimestamp(), ball.getCameraId(), ball.getPos()
			// .x(), ball.getPos().y(), corrBall.getPos().x(), corrBall.getPos().y()));
			
			List<Object> dataSet = new ArrayList<Object>();
			dataSet.add(System.nanoTime());
			dataSet.add(ball.getTimestamp());
			dataSet.add(ball.getCameraId());
			dataSet.add(ball.getPos().x());
			dataSet.add(ball.getPos().y());
			dataSet.add(0);
			dataSet.add(0);
			dataSet.add(0);
			dataSet.add(corrBall.getPos().x());
			dataSet.add(corrBall.getPos().y());
			dataSet.add(corrBall.getPos().z());
			exporter.addValues(dataSet);
			
			if ((i + 1) < balls.size())
			{
				clock.setNanoTime(balls.get(i + 1).getTimestamp());
			}
		}
		exporter.close();
	}
}
